@@ -170,7 +170,7 @@ void TableHead::mouseMoveEvent(QMouseEvent *e)
         QString s = htable->tipText(col);
 
         if (!s.isEmpty())
-            QToolTip::showText(e->globalPos(), s);
+            QToolTip::showText(e->globalPos(), s, this);
     }
 }
 
@@ -481,8 +481,6 @@ void TableBody::paintCell(QPainter *p, int row, int col)
 
     p->drawText(gap, 0, w, h, align, htable->text(row, col));
 
-    //htable->overpaintCell(p, row, col, gap); // senseless; Pstable::overpaintCell() should be removed
-
     // cache write!
     attr->text = htable->text(row, col);
     attr->xpos = htable->colXPos(col);
