@@ -70,7 +70,7 @@ class Lookup : public QObject
     Q_OBJECT
   public:
     Lookup();
-    ~Lookup();
+    ~Lookup() override;
     QString hostname(unsigned addr);
 
     static void initproctitle(char **argv, char **envp);
@@ -99,7 +99,7 @@ signals:
 
     QSocketNotifier *readsn, *writesn;
 
-    static const unsigned hostname_cache_size = 400; // max names to cache
+    static const int hostname_cache_size = 400; // max names to cache
 };
 
 #endif // LOOKUP_H
