@@ -64,7 +64,6 @@
 #define PROCDIR "/proc" // hmmm
 
 extern bool flag_thread_ok;
-extern bool flag_schedstat;
 extern bool flag_show_thread;
 
 int flag_24_ok; // we presume a kernel 2.4.x
@@ -155,6 +154,7 @@ char *read_proc_file(const char *fname, int pid = -1, int tgid = -1,
     if (fd < 0)
         return nullptr;
     r = read(fd, buffer_proc, sizeof(buffer_proc) - 1); // return 0 , -1 ,
+    close(fd);
     if (r < 0)
         return nullptr;
 
@@ -196,6 +196,7 @@ char *read_proc_file2(char *r_path, const char *fname, int *size = nullptr)
         return nullptr;
     r = read(fd, buffer_proc,
              sizeof(buffer_proc) - 1); // return 0 , -1 , read_count
+    close(fd);
     if (r < 0)
         return nullptr;
 
@@ -206,7 +207,6 @@ char *read_proc_file2(char *r_path, const char *fname, int *size = nullptr)
         *size = r;
 
     buffer_proc[r] = 0; // safer
-    close(fd);
 
     return buffer_proc;
     // note: not work  fgets(sbuf, sizeof(64), fp) why???
@@ -1215,7 +1215,7 @@ Procinfo::Procinfo(Proc *system_proc, int process_id, int thread_id) : refcnt(1)
     child_seq_prev = -1;
 
     lastchild = false;
-    generation = -1;
+    generation = 0;
     detail = nullptr;
 
     /// per_cpu_times = 0; not yet
@@ -1510,26 +1510,12 @@ int Procinfo::readproc()
         first_run = false;
     }
 
-    if (flag_schedstat == true)
-    {
-        // if no change then return.    twice faster !
-        // MAX_256 bytes check...?
-        // 2.6.9 upper only and some system no has
-        if ((sbuf = read_proc_file2(path, "schedstat")) == nullptr)
-            return -1;
-
-        if (hashcmp(sbuf))
-            return 1; // no change
-    }
     // read /proc/PID/stat
     if ((sbuf = read_proc_file2(path, "stat")) == nullptr)
         return -1;
 
-    if (flag_schedstat == false) // if no change then return.    twice faster !
-    {
-        if (hashcmp(sbuf))
-            return 1;
-    }
+    if (hashcmp(sbuf))
+        return 1;
 
     /*
             Not all values from /proc/#/stat are interesting; the ones left
@@ -1806,13 +1792,6 @@ int Proc::read_system() //
         else
             flag_thread_ok = false;
 
-        /* check schedstat  */
-        strcpy(path, "/proc/1/schedstat"); // some system doesn't have
-        if (!stat(path, (struct stat *)buf))
-            flag_schedstat = true;
-        else
-            flag_schedstat = false;
-
         strcpy(path, "/proc/stat");
         if ((n = read_file(path, buf, sizeof(buf) - 1)) <= 0)
             return 0;
