@@ -23,56 +23,47 @@
 #include <pwd.h>
 #include <grp.h>
 #include <sys/types.h>
-#include <cstdio>
-#include <cstring>
-#include <cstdlib>
 
 #include "uidstr.h"
 
-QHash<int, char *> Uidstr::udict;
-QHash<int, char *> Uidstr::gdict;
+QHash<int, QString> Uidstr::udict;
+QHash<int, QString> Uidstr::gdict;
 
 // return user name (possibly numeric)
-QString Uidstr::userName(int uid)
-{
-    char *p = udict.value(uid, NULL);
-    if (!p)
-    {
-        struct passwd *pw = getpwuid(uid);
-        if (!pw)
-        {
-            p = (char *)malloc(11);
-            sprintf(p, "%d", uid);
-        }
-        else
-            p = strdup(pw->pw_name);
-        udict.insert(uid, p);
-        // if(udict.count() > udict.size() * 3)
-        // udict.resize(udict.count());
+QString Uidstr::userName(int uid) {
+    auto it = udict.find(uid);
+    if (it != udict.end()) {
+        return it.value();
     }
-    QString s(p);
-    return s;
+
+    QString username;
+    struct passwd *pw = getpwuid(uid);
+    if (pw) {
+        username = QString::fromLocal8Bit(pw->pw_name);
+    } else {
+        username = QString::number(uid);
+    }
+
+    udict.insert(uid, username);
+    return username;
 }
 
 // return group name (possibly numeric)
 
-QString Uidstr::groupName(int gid)
-{
-    char *p = gdict[gid];
-    if (!p)
-    {
-        struct group *gr = getgrgid(gid);
-        if (!gr)
-        {
-            p = (char *)malloc(11);
-            sprintf(p, "%d", gid);
-        }
-        else
-            p = strdup(gr->gr_name);
-        gdict.insert(gid, p);
-        // if(gdict.count() > gdict.size() * 3)
-        // gdict.resize(gdict.count());
+QString Uidstr::groupName(int gid) {
+    auto it = gdict.find(gid);
+    if (it != gdict.end()) {
+        return it.value();
     }
-    QString s(p);
-    return s;
+
+    QString groupname;
+    struct group *gr = getgrgid(gid);
+    if (gr) {
+        groupname = QString::fromLocal8Bit(gr->gr_name);
+    } else {
+        groupname = QString::number(gid);
+    }
+
+    gdict.insert(gid, groupname);
+    return groupname;
 }
